"""
Test models
"""

# Standard Library
from unittest import mock

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import (
    DiscordPingTargets,
    FleetComm,
    FleetDoctrine,
    FleetType,
    FormupLocation,
    Webhook,
)


class TestModels(TestCase):
    """
    Test the models
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

    def test_discord_webhook_should_throw_exception(self):
        """
        Test if we get a ValidationError for a Discord webhook
        :return:
        """

        # given
        webhook = Webhook(
            url=(
                "https://discord.com/api/webhooks/754343402302911920F/x-BfFCdEG"
                "-qGg_39_mFwDU_vHdFUqRSLozaAxZdgKO2dm6Oa8vxNdAyesVpy-Bzf8pm-"
            ),
        )

        # when
        with self.assertRaises(ValidationError):
            webhook.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message=(
                "Invalid webhook URL. The webhook URL you entered does not match any "
                "known format for a Discord webhook. Please check the "
                "webhook URL."
            ),
        ):
            webhook.clean()

    @mock.patch("fleetpings.models.AA_FLEETPINGS_WEBHOOK_VERIFICATION", False)
    def test_discord_webhook_does_not_throw_exception(self):
        """
        Test we do not get a ValidationError for an invalid Discord webhook when
        AA_FLEETPINGS_WEBHOOK_VERIFICATION is set to False
        :return:
        """

        # given
        webhook = Webhook(url=("http://test/a/bad/webhook"))

        self.assertIsNone(webhook.clean())

    def test_doctrine_link_should_throw_exception(self):
        """
        Test if we get a ValidationError for a doctrine link
        :return:
        """

        # given
        doctrine = FleetDoctrine(
            name="Awesome Doctrine",
            link=("htp://invalid-doctrine.url"),
        )

        # when
        with self.assertRaises(ValidationError):
            doctrine.clean()

        with self.assertRaisesMessage(
            ValidationError,
            expected_message=("Your doctrine URL is not valid."),
        ):
            doctrine.clean()

    # def test_discord_ping_target_should_throw_not_synced_yet_exception(self):
    #     """
    #     Test if we get a ValidationError for a doctrine link
    #     :return:
    #     """
    #
    #     # given
    #     ping_target = DiscordPingTargets(name=self.group)
    #
    #     # when
    #     with self.assertRaises(ValidationError):
    #         ping_target.clean()
    #
    #     with self.assertRaisesMessage(
    #         ValidationError,
    #         expected_message=("This group has not been synced to Discord yet."),
    #     ):
    #         ping_target.clean()

    def test_should_return_fleetcomm_model_string_name(self):
        """
        Test should return the FleetComm model string name
        :return:
        :rtype:
        """

        test_object = FleetComm(name="Alliance Mumble")

        test_object.save()

        self.assertEqual(str(test_object), "Alliance Mumble")

    def test_should_return_fleetdoctrine_model_string_name(self):
        """
        Test should return the FleetDoctrine model string name
        :return:
        :rtype:
        """

        test_object = FleetDoctrine(name="Awesome Ships")

        test_object.save()

        self.assertEqual(str(test_object), "Awesome Ships")

    def test_should_return_formuplocation_model_string_name(self):
        """
        Test should return the FormupLocation model string name
        :return:
        :rtype:
        """

        test_object = FormupLocation(name="Alliance HQ")

        test_object.save()

        self.assertEqual(str(test_object), "Alliance HQ")

    def test_should_return_pingtarget_model_string_name(self):
        """
        Test should return the DiscordPingTargets model string name
        :return:
        :rtype:
        """

        test_object = DiscordPingTargets(name=self.group)

        self.assertEqual(str(test_object), self.group.name)

    def test_should_return_fleettype_model_string_name(self):
        """
        Test should return the FleetType model string name
        :return:
        :rtype:
        """

        test_object = FleetType(name="Fun Fleet")

        test_object.save()

        self.assertEqual(str(test_object), "Fun Fleet")

    def test_should_return_webhook_model_string_name(self):
        """
        Test should return the Webhook model string name
        :return:
        :rtype:
        """

        test_object = Webhook(
            name="Test Webhook",
            url=(
                "https://discord.com/api/webhooks/754119343402302920F/x-BfFCdEG"
                "-qGg_39_mFUqRSLoz2dm6Oa8vxNdaAxZdgKOAyesVpy-Bzf8wDU_vHdFpm-"
            ),
        )

        test_object.save()

        self.assertEqual(str(test_object), "Test Webhook")



"""
App setting
"""

# Django
from django.apps import apps
from django.core.handlers.wsgi import WSGIRequest

# AA Fleet Pings
from fleetpings.utils import clean_setting

# Set default panels if none are set in local.py
AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE = clean_setting(
    "AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE", False
)

# Allow non discord webhook url's
AA_FLEETPINGS_WEBHOOK_VERIFICATION = clean_setting(
    "AA_FLEETPINGS_WEBHOOK_VERIFICATION", True
)


def timezones_installed() -> bool:
    """
    Check if aa-timezones is installed
    :return: bool
    """

    return apps.is_installed("timezones")


def optimer_installed() -> bool:
    """
    Check if optimer_installed is installed
    :return: bool
    """

    return apps.is_installed("allianceauth.optimer")


def fittings_installed() -> bool:
    """
    Check if fittings is installed
    :return: bool
    """

    return apps.is_installed("fittings")


def discord_service_installed() -> bool:
    """
    Check if the Discord service is installed
    :return: bool
    """

    return apps.is_installed("allianceauth.services.modules.discord")


def srp_module_installed() -> bool:
    """
    Check if any of the SRP modules is installed
    :return:
    """

    return apps.is_installed("allianceauth.srp") or apps.is_installed("aasrp")


def srp_module_is(module_name: str) -> bool:
    """
    Check for a specific SRP module
    :param module_name:
    """

    return apps.is_installed(module_name)


def can_add_srp_links(request: WSGIRequest, module_name: str) -> bool:
    """
    Check if the current user has the rights to add SRP links for the module
    :param request:
    :param module_name:
    """

    return_value = False

    if module_name == "aasrp" and (
        request.user.has_perm("aasrp.manage_srp")
        or request.user.has_perm("aasrp.create_srp")
    ):
        return_value = True

    if module_name == "allianceauth.srp" and (
        request.user.has_perm("auth.srp_management")
        or request.user.has_perm("srp.add_srpfleetmain")
    ):
        return_value = True

    return return_value



# Django
from django.template import Context, Template
from django.test import TestCase

# AA Fleet Pings
from fleetpings import __version__


class TestVersionedStatic(TestCase):
    """
    Test fleetpings_versioned_static template tage
    """

    def test_versioned_static(self):
        """
        Test should return versioned static
        :return:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            "{% load fleetpings_versioned_static %}"
            "{% fleetpings_static 'fleetpings/css/fleetpings.min.css' %}"
        )

        rendered_template = template_to_render.render(context)

        self.assertInHTML(
            f'/static/fleetpings/css/fleetpings.min.css?v={context["version"]}',
            rendered_template,
        )



"""
Eve images
"""

# Alliance Auth
from allianceauth.eveonline.evelinks.eveimageserver import character_portrait_url
from allianceauth.eveonline.models import EveCharacter


def get_character_portrait_from_evecharacter(
    character: EveCharacter, size: int = 32, as_html: bool = False
) -> str:
    """
    Get the character portrait from EveCharacter model
    :param size:
    :param character:
    :param size:
    :param as_html:
    :return:
    """

    portrait_url = character_portrait_url(
        character_id=character.character_id, size=size
    )

    return_value = portrait_url

    if as_html is True:
        character_name = character.character_name
        return_value = (
            '<img class="aasrp-character-portrait img-rounded" '
            f'src="{portrait_url}" alt="{character_name}">'
        )

    return return_value



"""
URLs for render in templates

Uses: Allianceauth App Utils
"""

# Django
from django.template.defaulttags import register

# Alliance Auth (External Libs)
from app_utils.urls import reverse_absolute


@register.simple_tag
def fleetpings_reverse_url(view: str, *args) -> str:
    """
    Absolute URL
    :param view:
    :param args:
    :return:
    """

    return reverse_absolute(view, list(args))



"""
App config
"""

# Django
from django.apps import AppConfig

# AA Fleet Pings
from fleetpings import __version__


class AaFleetpingsConfig(AppConfig):
    """
    Application config
    """

    name = "fleetpings"
    label = "fleetpings"
    verbose_name = f"Fleet Pings v{__version__}"



"""
Pages url config
"""

# Django
from django.urls import include, path

# AA Fleet Pings
from fleetpings import views

app_name: str = "fleetpings"

urlpatterns = [
    path(route="", view=views.index, name="index"),
    # Ajax calls
    path(
        route="ajax/",
        view=include(
            [
                path(
                    route="create-fleet-ping/",
                    view=views.ajax_create_fleet_ping,
                    name="ajax_create_fleet_ping",
                ),
                path(
                    route="get-ping-targets-for-user/",
                    view=views.ajax_get_ping_targets,
                    name="ajax_get_ping_targets",
                ),
                path(
                    route="get-webhooks-for-user/",
                    view=views.ajax_get_webhooks,
                    name="ajax_get_webhooks",
                ),
                path(
                    route="get-fleet-types-for-user/",
                    view=views.ajax_get_fleet_types,
                    name="ajax_get_fleet_types",
                ),
                path(
                    route="get-formup-locations/",
                    view=views.ajax_get_formup_locations,
                    name="ajax_get_formup_locations",
                ),
                path(
                    route="get-fleet-comms/",
                    view=views.ajax_get_fleet_comms,
                    name="ajax_get_fleet_comms",
                ),
                path(
                    route="get-fleet-doctrines-for-user/",
                    view=views.ajax_get_fleet_doctrines,
                    name="ajax_get_fleet_doctrines",
                ),
            ]
        ),
    ),
]



"""
Handling Discord webhooks
"""

# Third Party
from dhooks_lite import Embed, Footer, Webhook

# Django
from django.contrib.auth.models import User
from django.utils import timezone

# AA Fleet Pings
from fleetpings.helper.eve_images import get_character_portrait_from_evecharacter
from fleetpings.helper.ping_context import _get_webhook_ping_context


def ping_discord_webhook(ping_context: dict, user: User):
    """
    Ping a Discord webhook
    :param ping_context:
    :return:
    """

    webhook_ping_context = _get_webhook_ping_context(ping_context=ping_context)

    discord_webhook = Webhook(ping_context["ping_channel"]["webhook"])
    message_to_send = webhook_ping_context["content"]
    embed_color = ping_context["ping_channel"]["embed_color"]
    author_eve_avatar = get_character_portrait_from_evecharacter(
        character=user.profile.main_character, size=256
    )
    author_eve_name = user.profile.main_character.character_name

    embed = Embed(
        description=message_to_send,
        title=".: Fleet Details :.",
        timestamp=timezone.now(),
        color=int(embed_color.lstrip("#"), 16),
        footer=Footer(f"Ping sent by by: {author_eve_name}", author_eve_avatar),
    )

    discord_webhook.execute(
        webhook_ping_context["header"], embeds=[embed], wait_for_response=True
    )



# Generated by Django 2.2.16 on 2020-09-15 16:17

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("fleetpings", "0003_fleetdoctrine_link"),
    ]

    operations = [
        migrations.AlterField(
            model_name="discordpingtargets",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) ...",
                related_name="discord_role_require_groups",
                to="auth.Group",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) ...",
                related_name="webhook_require_groups",
                to="auth.Group",
            ),
        ),
    ]
